"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = argufy;

/**
 *
 * @param {string[]} argv
 * @param {*} long
 * @param {*} short
 * @param {*} bool
 * @param {*} number
 */
const find = (argv, long, short, bool, number) => {
  const re = new RegExp(`^-(${short}|-${long})`);
  const i = argv.findIndex(a => re.test(a));
  if (i == -1) return {
    argv
  };

  if (bool) {
    return {
      value: true,
      argv: [...argv.slice(0, i), ...argv.slice(i + 1)]
    };
  }

  const j = i + 1;
  let value = argv[j];
  if (!value || typeof value == 'string' && value.startsWith('--')) return {
    argv
  };

  if (number) {
    value = parseInt(value, 10);
  }

  return {
    value,
    argv: [...argv.slice(0, i), ...argv.slice(j + 1)]
  };
};
/**
 * Parse the config and extract arguments from the `process.argv` array.
 * @param {Config} config The configuration for parsing, where each key is a flag and values are either strings, or objects with possible properties:
 *
 * - __short__ Shorthand for this argument, usually one letter.
 * - __boolean__ Does not have to be followed by a value, true if given.
 * - __number__ Parse as a number.
 * - __command__ Whether this argument can be specified as the first argument without the dash, such as `example run`.
 * - __multiple__ Extract multiple commands as an  array when `command` is set to true.
 *
 * @example
 *
 * const config = {
 *  title: 't',       // program -t Title
 *  open: {           // program -o
 *    short: 'o',
 *    boolean: true,
 *  },
 *  delay: {          // program -d 100
 *    short: 'd',
 *    number: true,
 *  },
 *  file: {           // program File.txt
 *    command: true,
 *  },
 *  file2: {           // program File.txt File2.txt
 *    command: true,
 *    multiple: true,
 *  },
 * }
 *
 * @param {string[]} [args] Array with arguments to parse. `process.argv` is used by default. It is assumed that user arguments start from the 3rd position.
 * @returns {Object.<string, string|string[]|boolean|number>} An object with all found values for the configuration request.
 */


function argufy(config = {}, args = process.argv) {
  let [,, ...argv] = args;
  /** @type {string} */

  const titles = findTitles(argv);
  argv = argv.slice(titles.length);
  const res = Object.keys(config).reduce(({
    _argv,
    ...acc
  }, key) => {
    if (_argv.length == 0) return {
      _argv,
      ...acc
    };
    const val = config[key];
    let value;

    if (typeof val == 'string') {
      ({
        value,
        argv: _argv
      } = find(_argv, key, val));
    } else {
      try {
        const {
          short,
          boolean,
          number,
          command,
          multiple
        } = val;

        if (command && multiple && titles.length) {
          value = titles;
        } else if (command && titles.length) {
          value = titles[0];
        } else {
          ({
            value,
            argv: _argv
          } = find(_argv, key, short, boolean, number));
        }
      } catch (err) {
        return {
          _argv,
          ...acc
        };
      }
    }

    if (value === undefined) return {
      _argv,
      ...acc
    };
    const r = {
      _argv,
      ...acc,
      [key]: value
    };
    return r;
  }, {
    _argv: argv
  });
  return res;
}

const findTitles = argv => {
  const titles = [];

  for (let i = 0; i < argv.length; i++) {
    const a = argv[i];
    if (a.startsWith('-')) break;
    titles.push(a);
  }

  return titles;
};
/**
 * @typedef {Object} Flag
 * @property {string} [short] Shorthand for this argument, usually one letter.
 * @property {boolean} [boolean] Does not have to be followed by a value, true if given.
 * @property {boolean} [number] Parse as a number.
 * @property {boolean} [command] Whether this argument can be specified as the first argument without the dash, such as `example run`: run is the command.
 * @property {boolean} [multiple] Extract multiple commands as an  array when `command` is set to true.
 *
 * @typedef {Object.<string, string|Flag>} Config
 */
//# sourceMappingURL=index.js.map